/**
 * I2C接続昇降圧コンバータ　TPS638811操作用
 */

#include <EEPROM.h>
#include "tps6381x.h"
tps6381x dcdc;

// VOUT 設定用
const int MAX_VOLT = 5200 ;  // 出力電圧の上限値
const int MIN_VOLT = 2500 ;  // 出力電圧の下限値：これ以下だと出力電圧示用のLED電圧計が動かなくなる
const int MAX_VOLT_DEF = 5000 ; // ボタン１を押したときの高圧側電圧
const int MIN_VOLT_DEF = 3300 ; // ボタン１を押したときの低圧側電圧

int volt = MAX_VOLT_DEF ; // 出力値の現在値
int voltMode = HIGH ;     // ボタン１を押したときの高圧側/低圧側切り替えフラグ

#define LEDPIN 1      // ボタン操作確認用LED
#define DATAADDR 0x00 // 電圧値を保存するEEPROMのアドレス

/**
 * 出力電圧値保存
 */
// EEPROMから初期設定値をLOAD
void loadEEPROMData( ) {
  EEPROM.get(DATAADDR, volt);

  //　最初期時(EEPROMに何も書き込んでいない場合)にトンデモな値が設定されるのを防ぐために出力値を補正
  if (volt < MIN_VOLT) { volt = MIN_VOLT ; }
  if (volt > MAX_VOLT) { volt = MAX_VOLT ; }
}

// EEPROMに初期設定値をSAVE
void saveEEPROMData( ) {
  EEPROM.put(DATAADDR, volt);
}

/**
 * 
 */
void setup() {
  pinMode(LEDPIN, OUTPUT);
  digitalWrite(LEDPIN, LOW);

  loadEEPROMData( ) ;
  dcdc.Range(HIGH);   // レンジ切り替え信号の意味が分からないので高圧側に固定
  dcdc.Out1(volt);
  dcdc.Enable(true);  // 出力値を設定してから出力ONにしないと多分不味いことが起こる
}

void loop() {
  // --- ボタンの読み取り
  int b1 = analogRead(3) ;
  int b2 = analogRead(4) ;
  int btn = 0 ;

  if (b2 > 128) {
    btn = 3 ;
    if (b2 > 768) { btn = 2 ;}
  }
  if (b1 > 128) { btn = 1 ;}

  // --- ボタンが押された場合の処理
  switch(btn) {
    case 1: // 高圧側/低圧側のディフォルト値に設定
      if (voltMode == HIGH) {
        voltMode = LOW ;
        volt = MIN_VOLT_DEF ;
        saveEEPROMData( ) ;
      } else {
        voltMode = HIGH ;
        volt = MAX_VOLT_DEF ;
        saveEEPROMData( ) ;
      }
      break ;

    case 2: // 出力電圧を+0.1V
      volt += 100 ;
      if (volt > MAX_VOLT) { volt = MAX_VOLT ; }
      saveEEPROMData( ) ;
      break ;

    case 3: // 出力電圧を-0.1V
      volt -= 100 ;
      if (volt < MIN_VOLT) { volt = MIN_VOLT ; }      saveEEPROMData( ) ;
      break ;
  }

  if (btn != 0) {
      dcdc.Out1(volt);
      // チャタリング防止のために0.5秒間停止
      digitalWrite(LEDPIN, HIGH);
      delay(500);
      digitalWrite(LEDPIN, LOW);
  }
}
